unit PerformanceCounter;

interface

uses
  SysUtils,
  Classes,
  Windows;

type
  TPerformanceCounter = class
  private
    FFrequency: int64;
  protected
    FGlobalMeasureStartTime: int64;
    FGlobalMeasureEndTime: int64;
    FMeasureRunning: boolean;
    CurrentMeasureDescription: string;
    CurrentMeasureStartTime: int64;
    MeasurementList: TList;
    function getGlobalRunTime: int64;
  public
    constructor Create();
    destructor Destroy(); override;

    procedure StartMeasure(Description: string);
    procedure StopMeasure();
    function toString(): string;

    property Frequency: int64 Read FFrequency;
    property MeasureRunning: boolean Read FMeasureRunning;
    property GlobalRunTime: int64 Read getGlobalRunTime;
  end;

  PMeasureEntry = ^TMeasureEntry;

  TMeasureEntry = record
    Description: string;
    RunTime:     int64;
  end;

var
  PC: TPerformanceCounter;

implementation

uses
  Math;

constructor TPerformanceCounter.Create();
begin
  inherited;
  QueryPerformanceFrequency(FFrequency);
  MeasurementList := TList.Create();
  FMeasureRunning := False;
  QueryPerformanceCounter(FGlobalMeasureStartTime);
  FGlobalMeasureEndTime := FGlobalMeasureStartTime;
end;

destructor TPerformanceCounter.Destroy();
var
  I: integer;
begin
  for I := 0 to MeasurementList.Count - 1 do
    Dispose(MeasurementList.Items[I]);
  MeasurementList.Free;
  inherited;
end;

procedure TPerformanceCounter.StartMeasure(Description: string);
begin
  if MeasureRunning then
    StopMeasure();
  FMeasureRunning := True;
  CurrentMeasureDescription := Description;
  QueryPerformanceCounter(CurrentMeasureStartTime);
  if (FGlobalMeasureStartTime = 0) then
    FGlobalMeasureStartTime := CurrentMeasureStartTime;
end;

procedure TPerformanceCounter.StopMeasure();
var
  EndTime:      int64;
  MeasureEntry: PMeasureEntry;
begin
  QueryPerformanceCounter(EndTime);
  New(MeasureEntry);
  MeasureEntry^.Description := CurrentMeasureDescription;
  MeasureEntry^.RunTime     := EndTime - CurrentMeasureStartTime;
  MeasurementList.Add(MeasureEntry);
  FMeasureRunning := False;
  FGlobalMeasureEndTime := EndTime;
end;

function TPerformanceCounter.getGlobalRunTime: int64;
begin
  Result := FGlobalMeasureEndTime - FGlobalMeasureStartTime;
end;

function TPerformanceCounter.toString(): string;

  function GetFloatLength(Value: extended): integer;
  var
    x: integer;
  begin
    Result := 1;
    x      := Trunc(Value);
    while (x > 10) do
    begin
      Inc(Result);
      x := x div 10;
    end;
  end;

const
  GlobalDescription = 'Overall time';
var
  I: integer;
  MeasureEntry: PMeasureEntry;
  MaxDescriptionLength: integer;
  dots, space: string;
  f: extended;
begin
  Result := '';
  MaxDescriptionLength := Length(GlobalDescription);
  for I := 0 to MeasurementList.Count - 1 do
  begin
    with PMeasureEntry(MeasurementList.Items[I])^ do
      MaxDescriptionLength := Max(MaxDescriptionLength, Length(Description));
  end;
  Inc(MaxDescriptionLength);
  for I := 0 to MeasurementList.Count - 1 do
  begin
    MeasureEntry := MeasurementList.Items[I];
    with MeasureEntry^ do
    begin
      //SetLength(s,);
      dots   := StringOfChar('.', MaxDescriptionLength - Length(Description));
      f      := RunTime * 1000 / frequency;
      space  := StringOfChar(' ', 5 - GetFloatLength(f));
      Result := Result + Format('%s:%s %s%3f ms%s',
        [Description, dots, space, f, #13#10]);
    end;
  end;
  Result := Result + #13#10;
  dots   := StringOfChar('.', MaxDescriptionLength - Length(GlobalDescription));
  f      := GlobalRunTime * 1000 div frequency;
  space  := StringOfChar(' ', 5 - GetFloatLength(f));
  Result := Result + Format('%s:%s %s%3f ms%s', [GlobalDescription,
    dots, space, f, #13#10]);
end;

begin
  PC := TPerformanceCounter.Create();
end.
